<?php
/**
 * Mai_Analyzer_Pattern
 * 
 * Copyright (C) 2009  Mai (Me Artificial Intelligence)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @author Wouter Bulten (wouterbulten@mai-ai.org)
 * @copyright Copyright (C) 2009  Mai (Me Artificial Intelligence)
 * @license http://opensource.org/licenses/gpl-3.0.html GNU General Public License version 3 (GPLv3)
 * @link http://www.launchpad.net/mai
 * 
 * @category Mai
 * @package Analyzer
 * @subpackge Pattern
 * 
 */

/** Load required file */
if(!Mai::getAutoloader())
{
	require_once 'Mai/Analyzer/Pattern/Interface.php';
}

/**
 * Mai_Analyzer_Pattern compares a pattern with given sources.
 * 
 * @author Wouter Bulten (wouterbulten@mai-ai.org)
 * @copyright Copyright (C) 2009  Mai (Me Artificial Intelligence)
 * @license http://opensource.org/licenses/gpl-3.0.html GNU General Public License version 3 (GPLv3)
 * @link http://www.launchpad.net/mai
 * 
 * @category Mai
 * @package Analyzer
 * @subpackage Pattern
 * 
 */

class Mai_Analyzer_Pattern implements Mai_Analyzer_Pattern_Interface
{
	/**
	 * @var Mai_Container_Sentence $_sentence A sentence container containing words and the pattern.
	 */
	private $_sentence;
	
	/**
	 * @var array $_sources An array with different input sources.
	 */
	private $_sources = array();
	
	/**
	 * @var Mai_Container_Output_Pattern $_outputData
	 */
	private $_outputData;
	
	/**
	 * Check if Mai has been setup and that the creator is Mai_Analyzer.
	 */
	public function __construct($creator)
	{
		if(!Mai::isInit())
    	{
    		trigger_error('Mai has not been setup yet. Please use Mai::init().', E_USER_ERROR);
    	}
    	
    	if(!($creator instanceof Mai_Analyzer) && !Mai_Config::getVar()->analyzer->allow_direct_access)
    	{
    		trigger_error('Please use an instance of Mai_Analyzer_Interface to create an instance of Mai_Analyzer_Pattern.', E_USER_NOTICE);
    	}
	}

	/**
	 * Analyze the pattern.
	 */
	public function run()
	{
		if(empty($this->_sentence))
		{
			trigger_error('No sentence set.', E_USER_NOTICE);
		}

		if(empty($this->_sources))
		{
			trigger_error('No sources set!', E_USER_NOTICE);
		}
		
		$matchFound = false;
		$matchData = array();
		
		//Base pattern is the fastest check
		$basePattern = $this->_sentence->getPattern();

		//loop through sources
		foreach($this->_sources as $name => $source)
		{
			//Loop through source data
			foreach($source as $key => $data)
			{
				//Check base pattern
				if($basePattern === $data['base-pattern'])
				{
					//Match found!
					$matchFound = true;
					
					//Store match
					$matchData[] = $data;
					
					//Log
					Mai_Log::info('Found pattern match for sentence \''. $this->_sentence->getSentenceString() .'\' from source: '.$name.'.');
					
					//Break from first loop
					break;
				}
			}
		}
		
		//Check if a match had been found and start further analysis
		if($matchFound)
		{
			//Reset
			$matchFound = false;
			
			//Get more info
			$timePattern = $this->_sentence->getTimePattern();
			$personPattern = $this->_sentence->getPersonPattern();
			
			//Loop through every match
			foreach($matchData as $key => $match)
			{
				if($timePattern === $data['time-pattern'] && $personPattern === $data['person-pattern'])
				{
					//Perfect match found
					$matchFound = true;
					
					$outputPatternContainer = Mai_Container::newContainerOutputPattern($this->_sentence);
					
					//Save data
					$outputPatternContainer->setBasePattern($data['answer-base-pattern']);
					$outputPatternContainer->setTimePattern($data['answer-time-pattern']);
					$outputPatternContainer->setPersonPattern($data['answer-person-pattern']);
					$outputPatternContainer->setOrderPattern($data['order-pattern'], $data['answer-order-pattern']);
			
					if($data['type'] === Mai::MSG_NORMAL_QUESTION)
					{
						$outputPatternContainer->setAsQuestion();
					}
					
					$this->_outputData = $outputPatternContainer;
					
					//Break from loop
					break;
				}
			}
		}
		else
		{
			//No match found, for testing show a message
			die('-SERVER-No pattern match found.');
		}
	}

	/**
	 * Set data to analyze.
	 * @param Main_Container_Sentence_Interface $data A sentence container. (Must implement Mai_Container_Sentence_Interface)
	 */
	public function set(Mai_Container_Sentence_Interface $data)
	{
		$this->_sentence = $data;
	}
	
	/**
	 * Add a pattern source to the analyzer.
	 * @param string $name
	 * @param array $data
	 */
	public function addSource($name, array $data)
	{
		if(empty($name) || !is_string($name))
		{
			throw new UnexpectedValueException('Name of source has not been set or is invalid!');
		}

		$this->_sources[$name] = $data;
	}

	/**
	 * Remove source.
	 * @param string $name
	 */
	public function removeSource($name)
	{
		if(array_key_exists($name, $this->_sources))
		{
			unset($this->_sources[$name]);
		}
		else
		{
			trigger_error('Source '.$name.' does not excist.', E_USER_NOTICE);
		}
	}
	
	/**
	 * Return result of run
	 * @return Mai_Container_Output_Pattern
	 */
	public function getOutputData()
	{
		return $this->_outputData;
	}
}