<?php
/**
 * Mai_Analyzer_Words
 * 
 * Copyright (C) 2009  Mai (Me Artificial Intelligence)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @author Wouter Bulten (wouterbulten@mai-ai.org)
 * @copyright Copyright (C) 2009  Mai (Me Artificial Intelligence)
 * @license http://opensource.org/licenses/gpl-3.0.html GNU General Public License version 3 (GPLv3)
 * @link http://www.launchpad.net/mai
 * 
 * @category Mai
 * @package Analyzer
 * @subpackage Words
 * 
 */

/**
 * Mai_Analyzer_Words is an sentence analyzer. It compares words with defined sources.
 * 
 * @author Wouter Bulten (wouterbulten@mai-ai.org)
 * @copyright Copyright (C) 2009  Mai (Me Artificial Intelligence)
 * @license http://opensource.org/licenses/gpl-3.0.html GNU General Public License version 3 (GPLv3)
 * @link http://www.launchpad.net/mai
 * 
 * @category Mai
 * @package Analyzer
 * @subpackage Words
 */

class Mai_Analyzer_Words implements Mai_Analyzer_Words_Interface
{
	/**
	 * @var array The word array to analyze.
	 */
	private $_wordArray = array();
	
	/**
	 * @var array Container classes
	 */
	private $_wordContainers = array();

	/**
	 * @var array Contains an array with data about the words.
	 */
	private $_wordArrayData = array();

	/**
	 * @var string Contains the punctuation mark.
	 */
	private $_sentenceMark;

	/**
	 * @var array An array with different input sources.
	 */
	private $_sources = array();
	
	/**
	 * @var int The type of storage (class or array)
	 */
	private $_storageType = Mai::STORAGE_TYPE_ARRAY;

	/**
	 * Check if Mai has been setup and that the creator is Mai_Analyzer.
	 */
	public function __construct($creator, $type = Mai::STORAGE_TYPE_ARRAY)
	{
		if(!Mai::isInit())
    	{
    		trigger_error('Mai has not been setup yet. Please use Mai::init().', E_USER_ERROR);
    	}
    	
    	if(!($creator instanceof Mai_Analyzer) && !Mai_Config::getVar()->analyzer->allow_direct_access)
    	{
    		trigger_error('Please use an instance of Mai_Analyzer to create an instance of Mai_Analyzer_Words.');
    	}
    	
    	if($type === Mai::STORAGE_TYPE_CLASS)
    	{
    		$this->_storageType = Mai::STORAGE_TYPE_CLASS;
    	}
   	}

	/**
	 * Analyze the word array (or containers) and save locally.
	 */
	public function run()
	{
		//wordArray must be valid
		if(empty($this->_wordArray) && empty($this->_wordContainers))
		{
			throw new UnexpectedValueException('No words to analyze!');
		}
		
		if(empty($this->_sources))
		{
			trigger_error('No sources set!', E_USER_NOTICE);
		}

		if($this->_storageType === Mai::STORAGE_TYPE_ARRAY)
		{
			//Clear data from previous runs
			unset($this->_wordArrayData);
			
			//Loop through every word and every source
			$wordId = 0;
			foreach($this->_wordArray as $key => $word)
			{
				foreach($this->_sources as $name => $data)
				{
					foreach($data as $dataName => $dataValue)
					{
						if($word === $dataValue['word'])
						{
							//Word has been found, save data.
							$this->_wordArrayData[$wordId] = $dataValue;
							Mai_Log::info('Word \''.$word.'\' found from source '.$name);
							
							//Break from data loop
							break;
							//Break from sources loop
							break;
						}
					}
				}
	
				//If the word is not found, the data will be empty
				if(!isset($this->_wordArrayData[$wordId]))
				{
					//Fill it with 0
					$this->_wordArrayData[$wordId] = 0;
				}
	
				//Go to next word
				$wordId++;
			}
		}
		elseif($this->_storageType === Mai::STORAGE_TYPE_CLASS)
		{
			//Loop through every word
			foreach($this->_wordContainers as $key => $word)
			{
				foreach($this->_sources as $name => $data)
				{
					foreach($data as $dataName => $dataValue)
					{
						if($word->getWord() === $dataValue['word'])
						{
							//Word has been found, save data.
							$word->setType($dataValue['wordtype']);
							//@Mai Todo: Uncomment ww_id
							$word->setWwId($dataValue['ww_id']);
							//@Mai Todo: Replaced by person?
							//$word->setTarget($dataValue['target']);
							$word->setTime($dataValue['time']);
							$word->setNumber($dataValue['number']);
							$word->setPerson($dataValue['person']);
							
							//Send message to log
							Mai_Log::info('Word \''.$word->getWord().'\' found from source '.$name);
							
							//Break from data loop
							break;
							//Break from sources loop
							break;
						}
					}
				}
			}
		}
	}

	/**
	 * Set the word array to analyze. This function uses references.
	 * Original data will be changed by this analyzer.
	 * @param int $type
	 * @param array $data
	 */
	public function set(array $data)
	{
		if($this->_storageType === Mai::STORAGE_TYPE_ARRAY)
		{
			$this->_wordArray = &$data;
		}
		elseif($this->_storageType === Mai::STORAGE_TYPE_CLASS)
		{
			$this->_wordContainers = &$data;
		}
		else
		{
			trigger_error('Unknown type.', E_USER_ERROR);
		}
	}

	/**
	 * Add a word source to the analyzer.
	 * @param string $name
	 * @param array $data
	 */
	public function addSource($name, array $data)
	{
		if(empty($name) || !is_string($name))
		{
			throw new UnexpectedValueException('Name of source has not been set or is invalid!');
		}

		//Sources can be empty, just log it and continue
		if(empty($data))
		{
			Mai_Log::info('Mai_Analyzer_Words: Source with name '.$name.' is empty.');
		}
		else
		{
			//Test the source
			//Test an element of the array (random)
			$test = $data[array_rand($data)];
			
			if(!isset($test['word']))
			{
				throw new UnexpectedValueException('Source with name '.$name.' is not valid. Each element should have a word assigned.');
			}
		}
		
		//Add
		$this->_sources[$name] = $data;
	}

	/**
	 * Remove source.
	 * @param string $name
	 */
	public function removeSource($name)
	{
		if(array_key_exists($name, $this->_sources))
		{
			unset($this->_sources[$name]);
		}
		else
		{
			trigger_error('Source '.$name.' does not excist.', E_USER_NOTICE);
		}
	}

	/**
	 * Return data from the analyzed words.
	 * This class also uses references, using this function is not required.
	 * @return array
	 */
	public function returnWordData()
	{
		if($this->_storageType === Mai::STORAGE_TYPE_ARRAY)
		{
			return $this->_wordArrayData;
		}
		elseif($this->_storageType === Mai::STORAGE_TYPE_CLASS)
		{
			return $this->_wordContainers;
		}
	}
}