<?php
/**
 * Mai_Container_Sentence
 * 
 * Copyright (C) 2009  Mai (Me Artificial Intelligence)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @author Wouter Bulten (wouterbulten@mai-ai.org)
 * @copyright Copyright (C) 2009  Mai (Me Artificial Intelligence)
 * @license http://opensource.org/licenses/gpl-3.0.html GNU General Public License version 3 (GPLv3)
 * @link http://www.launchpad.net/mai
 * 
 * @category Mai
 * @package Container
 * @subpackage Sentence
 * 
 */

/** Load required file */
if(!Mai::getAutoloader())
{
	require_once 'Mai/Container/Sentence/Interface.php';
}

/**
 * Mai_Container_Sentence is a container for a sentence.
 * 
 * @author Wouter Bulten (wouterbulten@mai-ai.org)
 * @copyright Copyright (C) 2009  Mai (Me Artificial Intelligence)
 * @license http://opensource.org/licenses/gpl-3.0.html GNU General Public License version 3 (GPLv3)
 * @link http://www.launchpad.net/mai
 * 
 * @category Mai
 * @package Container
 * @subpackage Sentence
 *
 */
class Mai_Container_Sentence implements Mai_Container_Sentence_Interface
{
	/**
	 * @var array $_words Storage for words.
	 */
	private $_words = array();
	
	/**
	 * @var String $_sentenceMark Sentence mark.
	 */
	private $_sentenceMark;
	
	/**
	 * @var array $_pattern Pattern of this sentence.
	 */
	private $_pattern = array();
	
	/**
	 * @var array $_timePattern Time pattern of this sentence
	 */
	private $_timePattern = array();
	
	/**
	 * @var array $_personPattern Person pattern of this sentence
	 */
	private $_personPattern = array();
	
	/**
	 * @var string $_sentenceString
	 */
	private $_sentenceString;
	
	/**
	 * 
	 * @param $creator Creator of this class
	 * @return void
	 */
	public function __construct($creator)
	{
		if(!($creator instanceof Mai_Container) && !Mai_Config::getVar()->container->allow_direct_access)
		{
			trigger_error('Please use Mai_Container to create a container class.', E_USESR_NOTICE);	
		}
	}
	
	/**
	 * Insert a word in the storage.
	 * @param Mai_Container_Word The word to store.
	 * @param $location The location (key) where to store.
	 */
	public function insertWord(Mai_Container_Word $word, $location = null)
	{
		if(!isset($location) || !is_numeric($location))
		{
			$this->_words[] = $word;
		}
		else
		{
			$this->_words[$location] = $word;
		}
	}
	
	/**
	 * Return a word from storage.
	 * @param Int $location The location (key) of storage.
	 * @param Bool $showError Set to false to return false if a word does not excist.
	 * @return Mai_Container_Word
	 */
	public function getWord($location, $showError = true)
	{
		if(!isset($location) || !is_numeric($location))
		{
			trigger_error('Location not set.', E_USER_ERROR);
		}
		
		if(!isset($this->_words[$location]) && $showError === true)
		{
			if($showError)
			{
				trigger_error('Word does not excist for location.', E_USER_NOTICE);
			}
			else
			{
				return false;
			}
		}
		
		return $this->_words[$location];
	}
	
	/**
	 * Return all words (by reference)
	 * @return array
	 */
	public function &getAll()
	{
		return $this->_words;	
	}
	
	/**
	 * Set the sentence mark of this sentence.
	 * @param String $sentenceMark
	 */
	public function setSentenceMark($sentenceMark)
	{
		if(!isset($sentenceMark) || empty($sentenceMark))
		{
			trigger_error('Sentence mark not set.', E_USER_ERROR);
		}
		
		return $this->_sentenceMark;
	}
	
	/**
	 * Return the sentence mark of this sentence.
	 * @return String
	 */
	public function getSentenceMark()
	{
		return $this->_sentenceMark;
	}
	
	/**
	 * Return the pattern of this sentence.
	 * @return array
	 */
	public function getPattern()
	{
		if(empty($this->_pattern))
		{
			$this->createPattern();
		}
		
		return $this->_pattern;
	}

	/**
	 * Create pattern of this sentence.
	 * @return void
	 */
	public function createPattern()
	{
		//Empty current data
		$this->_pattern = array();

		//Create a string version of the pattern to send to the log
		$patternString = '';
		
		foreach($this->_words as $key => $value)
		{
			$this->_pattern[$key] = $value->getType();
			
			//First element should not have a -
			if($patternString === '')
			{
				$patternString = (string) $value->getType();
			}
			else
			{
				$patternString = (string) $patternString . '-' . $value->getType();
			}
		}
		
		//Send to log
		Mai_Log::info('Created pattern for sentence \''.$this->getSentenceString().'\': '.$patternString);
	}
	
	/**
	 * Set a string version of this sentence.
	 * @param string $sentence
	 */
	public function setSentenceString($sentence)
	{
		if(empty($sentence))
		{
			trigger_error('Sentence must be set.', E_USER_ERROR);
		}
		
		$this->_sentenceString = $sentence;
	}
	
	/**
	 * Get the string version of this sentence.
	 * @return string
	 */
	public function getSentenceString()
	{
		if(empty($this->_sentenceString))
		{
			$this->createSentenceString();
		}
		
		return $this->_sentenceString;
	}
	
	/**
	 * Create a string version of this sentence.
	 * @return void
	 */
	private function createSentenceString()
	{
		$sentence = '';
		
		foreach($this->_words as $key => $word)
		{
			$sentence .= $word->getWord() . ' ';
		}
		
		//Trim whitespace
		$this->_sentenceString = trim($sentence);
	}
	
	/**
	 * Get time pattern of this sentence.
	 * @return array
	 */
	public function getTimePattern()
	{
		if(empty($this->_timePattern))
		{
			$this->createTimePattern();
		}
		
		return $this->_timePattern;
	}
	
	/**
	 * Create the time pattern of this sentence
	 * @return void
	 */
	private function createTimePattern()
	{
		foreach($this->_words as $key => $word)
		{
			$this->_timePattern[$key] = $word->getTime();
		}
	}
	
	/**
	 * Get person pattern of this sentence.
	 * @return array
	 */
	public function getPersonPattern()
	{
		if(empty($this->_personPattern))
		{
			$this->createPersonPattern();
		}
		
		return $this->_personPattern;
	}
	
	/**
	 * Create the person pattern of this sentence
	 * @return void
	 */
	private function createPersonPattern()
	{
		foreach($this->_words as $key => $word)
		{
			$this->_personPattern[$key] = $word->getPerson();
		}
	}
}