<?php
/**
 * Mai_Word_Splitter
 * 
 * Copyright (C) 2009  Mai (Me Artificial Intelligence)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @author Wouter Bulten (wouterbulten@mai-ai.org)
 * @copyright Copyright (C) 2009  Mai (Me Artificial Intelligence)
 * @license http://opensource.org/licenses/gpl-3.0.html GNU General Public License version 3 (GPLv3)
 * @link http://www.launchpad.net/mai
 * 
 * @category Mai
 * @package Word
 * @subpackage Splitter
 */

/** Load required file */
if(!Mai::getAutoloader())
{
	require_once 'Mai/Container/Sentence.php';
	require_once 'Mai/Container/Word.php';
}

/**
 * Mai_Word_Splitter splits a string in to smaller pieces.
 * 
 * @author Wouter Bulten (wouterbulten@mai-ai.org)
 * @copyright Copyright (C) 2009  Mai (Me Artificial Intelligence)
 * @license http://opensource.org/licenses/gpl-3.0.html GNU General Public License version 3 (GPLv3)
 * @link http://www.launchpad.net/mai
 * 
 * @category Mai
 * @package Word
 * @subpackage Splitter
 *  
 */
 
class Mai_Word_Splitter implements IteratorAggregate
{
	/**
	 * The string to split.
	 * @var string
	 */
	protected $_wordsString;
	
	/**
	 * Contains an array with words, sorted by sentence.
	 * @var array
	 */
	protected $_wordArray = array();
	
	/**
	 * @var array An array with container classes. Used when saving to classes.
	 */
	protected $_containerArray = array();
	
	/**
	 * Contains the punctuation marks, sorted by sentence
	 * @var array
	 */
	protected $_sentenceMarks = array();
	
	/**
	 * Contains the number of sentences.
	 * @var integer
	 */
	protected $_sentences = 0;
	
	/**
	 * Punctuation marks that mark a new sentence.
	 * @var array
	 */
	protected $_newSentenceMarks = array('!', '.', ',', '?');
	
	/**
	 * Type of storage (whether to use classes or not)
	 * @var int
	 */
	protected $_type = Mai::STORAGE_TYPE_ARRAY;

	/**
	 * Constructor checks if Mai has been set up.
	 */
	public function __construct($type = Mai::STORAGE_TYPE_ARRAY)
	{
		if(!Mai::isInit())
    	{
    		trigger_error('Mai has not been setup yet. Please use Mai::init().', E_USER_ERROR);
    	}
    	
    	//Store the type
    	if($type === Mai::STORAGE_TYPE_CLASS)
    	{
    		$this->_type = Mai::STORAGE_TYPE_CLASS;
    	}
	}

	public function setString($string)
	{
		if(!isset($string) || empty($string))
		{
			throw new InvalidArgumentException('String is not valid.');
		}
		
		$this->_wordsString = $string;
	}
	
	/**
	 * Runs the splitting process.
	 * @param Int $type Type of storage
	 * @return void
	 */
	public function run()
	{
		if(!isset($this->_wordsString) || empty($this->_wordsString))
		{
			trigger_error('String is not valid.', E_USER_ERROR);
		}
		
		//Split the string
		$explodedString = explode(' ', $this->_wordsString);
		//Amount of sentences
		$sentences = 0;
		//Contains the words, sorted by sentence
		$wordArray = array();
		//The punctuation marks, sorted by sentence
		$sentenceMarks = array();
		
		foreach($explodedString as $key => $value)
		{
			//Look for the start of a new sentence
			if(in_array($value{(strlen($value) - 1)}, $this->_newSentenceMarks))
			{
				//Store the mark
				$sentenceMarks[$sentences] = $value{(strlen($value) - 1)};
				//Remove it from the word
				$value = substr($value, 0, -1);
				//Store the value
				$wordArray[$sentences][] = $value;
				//Increase the amount (let new words start in a new sentence)
				$sentences++;
				
			}
			else
			{
				//Store the value
				$wordArray[$sentences][] = $value;

				//The last sentence could not have a sentence mark.
				//Then we have to increase the sentences counter.
				$lastWordKey = count($explodedString) - 1;
				//Only do this when we have reached the last word
				if($key == $lastWordKey)
				{
					$sentences++;
				}
			}			
		}
		
		//Store in local variables
		$this->_wordArray = $wordArray;
		$this->_sentenceMarks = $sentenceMarks;
		$this->_sentences = $sentences;
		
		//Words can also be stored in containers.
		if($this->_type === Mai::STORAGE_TYPE_CLASS)
		{
			foreach($this->_wordArray as $key => $sentence)
			{
				$sentenceContainer = Mai_Container::newContainerSentence();
				
				foreach($sentence as $wordKey => $word)
				{
					//Create a new instance of Mai_Container_Word and store.
					 $sentenceContainer->insertWord(Mai_Container::newContainerWord($word));
				}
				
				//Store sentence mark
				if(isset($this->_sentenceMarks[$key]))
				{
					$sentenceContainer->setSentenceMark($this->_sentenceMarks[$key]);
				}
				
				//Store container local
				$this->_containerArray[$key] = $sentenceContainer;
			}
		}
	}
	
	/**
	 * Return the word array. Is empty if called before run().
	 * @return array
	 */
	public function returnWordArray()
	{
		return $this->_wordArray;
	}
	
	/**
	 * Return the container array. Is empty if called before run().
	 * @return array
	 */
	public function returnContainerArray()
	{
		return $this->_containerArray;
	}
	
	/**
	 * Return the sentence marks array. Is empty if called before run().
	 * @return array
	 */
	public function returnSentenceMarks()
	{
		return $this->_sentenceMarks;
	}
	
	/**
	 * Return the amount of sentences.
	 * @return integer
	 */
	public function returnSentencesAmount()
	{
		return $this->_sentences;
	}
	
	/**
	 * Function to implement the IteratorAggregate interface.
	 * @return ArrayIterator
	 */
	public function getIterator()
	{
		if($this->_type === Mai::STORAGE_TYPE_CLASS)
		{
			return new ArrayIterator($this->_containerArray);
		}
		
		return new ArrayIterator($this->_wordArray);
	}
}